/*
    Tucnak - VHF contest log
    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

#ifndef __CYGWIN__

struct ntpq *ntpq = NULL;

struct ntpq *init_ntpq(){
	struct ntpq *ntpq;

	ntpq = g_new0(struct ntpq, 1);
	ntpq->buf = g_string_sized_new(1024);
	ntpq->good = 1;
	ntpq->timer = install_timer(1000, ntpq_timer, (union cba_t)ntpq);
	return ntpq;
}

void free_ntpq(struct ntpq *ntpq){
	if (ntpq->pid) kill(ntpq->pid, SIGTERM);
	if (ntpq->rfd>=0) close(ntpq->rfd);
	if (ntpq->timer) kill_timer(ntpq->timer);
	g_free(ntpq);
}

void ntpq_timer(union cba_t cba){
    int fds[2], i;
	struct ntpq *ntpq;
    char errbuf[1024];
	
    /*dbg("ntpq_timer()\n");*/
	ntpq=cba.ntpq;
	ntpq->timer=0;
	pipe(fds);
/*	dbg("pipe=%d %d\n", fds[0], fds[1]);*/
	ntpq->rfd=fds[0];
    ntpq->pid=fork();
    if (ntpq->pid<0) {
        dbg("Can't fork for ntpq\n");
        return;
    }
    if (ntpq->pid==0){
        /* child */
        setenv("TERM", "dumb", 1);
        for (i=0;i<1024;i++) {
			if (i==fds[1]) continue;
			close(i);
		}
		open("/dev/null", O_RDONLY);
		dup(fds[1]);
		dup(fds[1]);
        execlp("ntpq", "ntpq", "-pn", NULL);
        fprintf(stderr, " exec failed errno=%d %s\n", errno, strerror_r(errno, errbuf, sizeof(errbuf)));
        exit(-1);
    }                  
    /* parent */
    ntpq->rfd=fds[0];
	close(fds[1]);
    set_handlers(ntpq->rfd, ntpq_read_handler, NULL, NULL, (union cba_t)ntpq);
}


void ntpq_read_handler(union cba_t cba){
    char s[1030];
    int ret,err, good;
    gchar *c, *d;
	struct ntpq *ntpq;

	ntpq=cba.ntpq;

/*	dbg("read...\n");*/
    ret=read(ntpq->rfd, s, 1024);
/*	dbg("read=%d\n", ret);*/
    err=errno;
	if (ret<0){
/*        dbg("ntpq read error\n");*/
		set_handlers(ntpq->rfd, NULL, NULL, NULL, CBA0);
		close(ntpq->rfd);
		ntpq->timer = install_timer(50001, ntpq_timer, (union cba_t)ntpq);
		return;
	}
    if (ret==0){
        good=0;
        for (d=ntpq->buf->str; *d!='\0'; d++){
            if (*d=='*') {
				safe_strncpy(s, d+1, 10);
				c=index(s, '.');
				if (!c) break;
				*c='\0';
				if (strcmp(s, "127")==0) break; /* local clock */
                good=1;
                break;
            }
            for (;*d!='\0';d++){
                if (*d!='\n') continue;
                break;
            }
        }
		if (ctest && !ctest->oldcontest && ntpq->oldgood!=good) {
			ntpq->oldgood=good;
			if (!good){
				log_addf("Warning! Time on this machine is not synchronized. Please use NTP");
			}
		}

		ntpq->good=good;
        /*dbg("ntpq->good=%d\n", ntpq->good);*/
        g_string_truncate(ntpq->buf, 0);
		set_handlers(ntpq->rfd, NULL, NULL, NULL, CBA0);
		close(ntpq->rfd);
		if (good)
			ntpq->timer = install_timer(30*60*1000, ntpq_timer, (union cba_t)ntpq);
		else
			ntpq->timer = install_timer(2*60*1000, ntpq_timer, (union cba_t)ntpq);
		return;
    }
	
    s[ret]='\0';
    /*dbg("ntpq_read_handler('%s');\n", s);*/
    g_string_append(ntpq->buf, s);
}

#endif
